(* mpairs.m*)

(* Version 1 *)
 
(*Created by Rolf Sulanke, November 12, 2003.*)   
  
(*The package contains definitions and procedures for the euclidean and
the Moebius geometry of pairs of subspheres in the 3-sphere. It continues 
the packages euvec.m which contains concepts of the euclidean 
geometry of spheres. Operations and functions of pseudo-euclidean vector \
algebra (see the package pseuvec.m and the notebook pseuklid.m) are used. 
Pairs of spheres are treated in the notebook mspheres.nb and the package  
mspher.m, pairs of circles in the notebook mcircles.nb and the package mcirc.m. 
The current package is tested, explained, and  applied in the notebook 
pairs.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(* Hint: Before importing this package be sure that the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space are \
defined as constants within the Global context. *)

(* ATTENTION! Many of the functions defined in this package are usable for \
the 3-dimensional Moebius space, i.i. the 3-sphere, only! Therefore the setting 
dim = 5, ind = 1 is strongly recommended. *)

BeginPackage["mpairs`", {"mcirc`", "mspher`","euvec`","pseuvec`"}] 

moebsc::usage = "If vn is a spacelike unit vector defining a hypersphere S
(sphere, if dim = 5), and {cn1,cn2} is an orthonormed pair of vectors defining 
a (dim - 3)-sphere C (a circle, if dim = 5), then moebsc[vn,cn1,cn2] is the main 
Moebius invariant of the pair(S,C) defining it up to Moebius transforms. 
WARNING: if vn is not normalized, or (cn1, cn2) not an orthonormal pair, the number 
moebsc[vn,cn1,cn2] has no Moebius invariant meaning.";

euklidmoebsc::usage = "euklidmoebsc[d,r,R,phi] is the Mbius invariant moebsc of a 
sphere with radius r and a circle of radius R in the 3-dimensional euclidean space; 
d denotes the distance of the centers, and phi, 0<=phi<Pi, the angle between the line 
joining the centers and the normal of the plane through the circle; phi=0 if d=0."; 

planecirclemoebsc::usage = "planecirclemoebsc[d,R,alpha] is the Mbius invariant moebsc 
of a plane and a circle of radius R in the 3-dimensional euclidean space; d denotes the 
distance of the center of the circle to the plane, and alpha, 0<=alpha<Pi, the angle 
between the given plane and the plane through the circle."; 

pp2space::usage = "If a and b are the coordinate tupels of two distinct points of the 
euclidean n-space with respect to an orthogonal cartesian coordinate system, then 
pp2space[a_List,b_List] is a pseudo-orthonormal basis of the pseudo-euclidean 2-subspace 
of the (n+2)-dimensional pseudo-euclidean vector space whose isotropic subspaces represent 
the point pair (a,b).";

vspacepp::usage = "vspacepp[a_List,b_List] is an orthonormal basis of the orthogonal 
complement of the pseudo-euclidean 2-subspace pp2space[a_List,b_List].";

psframepp::usage = "If a and b are the coordinate tupels of two distinct points of the 
euclidean n-space with respect to an orthogonal cartesian coordinate system, then 
fpp=psframepp[a_List,b_List] is a pseudo-orthonormal basis of the pseudo-euclidean  
(n+2)-dimensional vector space such that the isotropic vectors fpp[[n+1]]+fpp[[n+2]] and 
fpp[[n+1]]-fpp[[n+2]] represent the points a, and b, respectively.";

indicator::usage = "indicator[vn,p1, p2] is the null vector, iff the 0-sphere S0=(p1,p2) 
of the n-dimensional euclidean space is contained in the hypersphere S=S(vn) defined by the 
spacelike vector vn of the (n+2)-dimensional pseudo-euclidean space, it is an isotropic 
vector iff only one of the points p1,p2 belongs to S, it is timelike, iff the hypersphere S 
separates the points (p1,p2), and spacelike, if these points lie on the same side of S."

moebspp::usage = "If vn is a spacelike vector defining a hypersphere S
(sphere, if dim = 5), and {a, b} is a pair of points, a != b, defining 
a 0-sphere S0 of the euclidean space, then moebspp[vn,a,b] is the main 
Moebius invariant of the pair(S,S0) defining it up to Moebius transforms.";

euklidmoebspp::usage = "euklidmoebspp[r,t,d,u] is the Mbius invariant moebspp of a 
hypersphere with radius r and a 0-sphere of radius t in the n-dimensional euclidean space; 
d denotes the distance of the centers, and u, 0<=u<Pi, the angle between the line 
joining the centers and the line containing the 0-sphere; u=0 if d=0."; 

circlepointsphere::usage = "circlepointsphere[cn1,cn2,p] yields a spaselike unit vector 
defining the sphere, which contains the circle C defined by the euclidean 2-space spanned by 
the spacelike 5-vectors cn1,cn2 of the pseudo-euclidean 5-space, and the point p of the 
euclidean 3-space; if p belongs to C then circlepointsphere returns the null vector null[5]."



dimm = Global`dim;

indd = Global`ind;
  
Begin["`Private`"]
 
 (* The main Moebius invariant of a pair (Sphere, Circle). *)
 
 moebsc[vn_,cn1_,cn2_]:= pssp[vn, cn1]^2 +  pssp[vn, cn2]^2
 
 
 (* moebsc expressed in euclidean terms. *)
 
 euklidmoebsc[d_,r_,R_, phi_] := 
        ((d^2 -r^2 -R^2)/(2*r*R))^2 + ((d*Cos[phi])/r)^2
    
 planecirclemoebsc[d_,R_,alpha_] := (d/R)^2 +(Cos[alpha])^2
 
 (* The pseudo-euclidean 2-space corresponding to a point pair is the spaan of *)
 
 pp2space[a_List,b_List] := Module[{eu,d = Length[a],m},
		If[(Length[b] != d)||(a== b), 
				Print["Incorrect arguments!"];Return[]];
    m = Table[null[d+2] ,{2}];
    		eu =4(a-b).(a-b)/((1 + a.a)*(1 + b.b));
    	m[[1]] = (invstepro[a] - invstepro[b])/Sqrt[eu];
    m[[2]]= (invstepro[a] + invstepro[b])/Sqrt[eu]; Return[m]]

pp2space[a_List] := pp2space[a[[1]], a[[2]]]

(*The orthogonal complement of pp2space[a,b] has the orthonormal basis *)

vspacepp[a_List, b_List] := Module[{m,m1, d = Length[a],prod,u,v},
    m = pp2space[a,b];
    prod[u_,v_]:=pssp[u,v,d+2,1];
    If[m \[Equal] Null, Return[Null]];
    m1 = orthonorm[NullSpace[{dual[m[[1]],d+2],dual[m[[2]],d+2]}], 
    innerprod -> prod];
    Return[m1]]
vspacepp[a_List] := vspacepp[a[[1]], a[[2]]]

(*An adapted pseudoorthonormal frame of a point pair: *)

psframepp[a_List,b_List] :=  Module[{m,m1, d = Length[a],prod,u,v},
    m = pp2space[a,b];
    If[m \[Equal]Null, Return[Null]];
    prod[u_,v_]:=pssp[u,v,d+2,1];
    m1 = orthonorm[NullSpace[{dual[m[[1]],d+2],dual[m[[2]],d+2]}],
    innerprod -> prod];
    Return[Join[m1,m]]]
    
 (* the indicator of the mutual position of a hypersphere and a point pair *)
 
 indicator[n_List,a_List,b_List] := pscross[Join[ vspacepp[a,b],{n}]]
    
 (* moebspp: The main Moebius invariant of a pair (hypersphere, point pair). *) 
    
 moebspp[n_List,a_List,b_List] := Module[{m, dn = Length[n], da=Length[a]},
    If[(da \[NotEqual] Length[b])|| (da+2 \[NotEqual] dn), 
      Print["Incorrect arguments!"];Return[]];
    m = vspacepp[a,b];
    Return[Sum[pssp[m[[i]], n,dn]^2,{i,da}]/pssp[n,n,dn]]]
    
  (* moebspp expressed in euclidean terms. *)
    
 euklidmoebspp[r_,t_,d_,u_] := (d^2-r^2-t^2)^2/(4*r^2*t^2) + (d*Sin[u]/r)^2
 
 (* sphere through a given circle and a given point *)
 
 circlepointsphere[cn1_,cn2_,p_]:=Module[{vc},
    vc =pscomplement[cn1,cn2];
    vc = AppendTo[vc,invstepro[p]];
    Return[normalize[pscross[vc]]]] 

End[]
  
  
Protect[moebsc, euklidmoebsc, planecirclemoebsc, pp2space, vspacepp, psframepp, 
indicator, moebspp,euklidmoebspp, circlepointsphere]


EndPackage[]


