(* riemann.m *)


(* Version 1 *)
 
(*Created by Rolf Sulanke, April 30, 2004. *)   
  
(*The package contains definitions and procedures for the complex and real 
Moebius geometry of the 2-sphere, considered as the complex Riemannian sphere or 
the elliptic quadric in the 3-dimensional real projective space. It needs the packages
mcirc.m, mspher.m, pseuvec.m, and euvec.m, containing Mathematica functions and tools 
for the euclidean and Moebius geometry of spheres. Operations and functions of 
pseudo-euclidean vector algebra (see the package pseuvec.m and the notebook pseuklid.m) are used. 
The package is tested, explained, and  applied in the notebooks riemsph.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres", this package and the notebbook riemsph.nb. 
Please, cite the source and notice the changes, or new concepts,  which you introduced.*)

(* ATTENTION! Many of the functions defined in this package are usable for the 
2-dimensional Moebius sphere or the Riemannian sphere only! Therefore the dimension dim
and the index ind must be set as dim = 4, ind = 1. *)

BeginPackage["riemann`", {"euvec`", "pseuvec`", "mspher`", "mcirc`"}]

infy::usage = "infy = ComplexInfinity is used as abbreviation. The message \
Power::infy is switched off.";

isotrop1::usage = "isotrop1[z] is the isotropic 4-vector with 4-th component \
1 in the pseudo-euclidean vector space representing the point z of the \
Riemannian sphere.";

isotrop::usage = "isotrop[z] is an isotropic 4-vector (without norming the \
4-th component) in the pseudo-euclidean vector space representing the point z \
of the Riemannian sphere.";

invisotrop::usage = "invisotrop[a_List] is the inverse of the function isotrop; it 
gives the element z of the Riemannian sphere corresponding to the isotropic 4-vector a."; 


rs2::usage = 
    "rs2[z] is the coordinate 3-vector of the point of the unit sphere \
corresponding to the complex number z.";

rs2point::usage = 
    "rs2point[z] is the Graphics3D-object, the point of the unit sphere corresponding \
to the number z of the Riemannian sphere, rs2point is Listable.";

rs2pointplot::usage = 
    "rs2pointplot[z,d:0.01] plots the point of the unit sphere corresponding \
to the number z of the Riemannian sphere with PointSize d, Default d = 0.01. \
rs2pointplot is Listable.";

g0::usage = 
    "g0[a,b,c,d][z] is the image of z under the orientation preserving M\
bius transform with parameters a,b,c,d.";

g1::usage = 
    "g1[a,b,c,d][z] is the image of z under the orientation reversing M\
bius transform with parameters a,b,c,d.";

crossratio::usage = 
    "crossratio[{p1,p2},{p3,p4}] is the crossratio of the throw \
(p1,p2,p3,p4).";

crvalues::usage = 
    "crvalues[z] is the List of the at most 6 different values of the \
crossratio of a throw of points with cross ratio z under all the 24 \
permutations of the points.";

scale::usage = 
    "scale[b,c,d][z] is the projective scale value of z on the projective \
line with points b->1, c->0, d->Infinity.";

rspsframepp::usage = 
    "rspsframepp[z1,z2] is an adapted orthonormal frame of the \
pseudo-euclidean 4-space the isotropic vectors of the 2-space spanned by the \
last two vectors of the frame correspond to the points z1,z2 of the \
Riemannian sphere.";

rsvspacepp::usage = 
    "rsvspacepp[z1,z2] yields an orthonormal basis of the orthogonal complement 
of the pseudo-euclidean 2-space in the 4-space spanned by the vectors corresponding 
to the points z1,z2 of the Riemannian sphere.";

rsoperatorpp::usage = 
    "rsoperatorpp[{a1,a2},{b1,b2}] is the selfadjoint operator of the double \
projection corresponding to the 0-spheres {a1,a2},{b1,b2} of the Riemannian \
sphere.";

rscenter3pts::usage = 
    "rscenter3pts[a1,a2,a3] is the center of the circle through a1,a2,a3 in \
the complex plane.";

rsradius3pts::usage = 
    "rsradius3pts[a1,a2,a3] is the radius of the circle through a1,a2,a3 in \
the complex plane.";

rscircle3pts ::usage = "rscircle3pts[a1,a2,a3][t] is a parameter representation of 
the circle or line through the three points a1,a2,a3 of the complex plane with 
real parameter t."

rsplotcircle3pts::usage = 
    "rsplotcircle3pts[a1,a2,a3, x:0, y:2*Pi, opts] plots the circle arc \
between the angles x,y (or the line) through the points a1,a2,a3 in the \
complex plane; opts may be options of ParametricPlot.";

rsvector3pts::usage = 
    "rsvector3pts[z1,z2,z3] returns a spacelike vector of the 4-dimensional \
pseudo-euclidean vector space corresponding to the circle or line through the \
3 different points z1,z2,z3 of the Riemannian sphere.";

rscirclevector::usage = 
    "rscirclevector[m,r] is the spacelike unit vector in the 4-space \
corresponding to the circle with center m and radius r in the complex plane]";

rsvcenter::usage = 
    "rsvcenter[alist] gives the complex number beeing the center of the \
circle which corresponds to the spacelike 4-vector alist.";

rsCoxeter::usage = 
    "rsCoxeter[m,m1,r,r1] is the Coxeter invariant of two circles with centers \
m, m1, and radii r, r1 in the complex plane.";

sl2Ctoso31::usage = 
    "sl2Ctoso31[a,b,c,d] yields a pseudo-orthonormal matrix function with \
values in SO(3,1)+, describing a homomorphism SL(2,C) -> SO(3,1)+. Warning: \
This is true for arguments a,b,c,d with Det[{{a,b},{c,d}}] = 1 only!";

 Begin["`Private`"] 

(* The complex infinity (abbreviation) *)

Off[Power::infy]
infy := ComplexInfinity;

(* The transition functions *)

isotrop1[z_] := Module[{x, y, a},
    If[z == Null, Print["Argument of isotrop1 is Null!"]; Return[]];
    If[z == ComplexInfinity, Return[{0, 0, 1, 1}]];
    x = Re[z];
     y = Im[z];
    a = Abs[z]^2;
    Return[FullSimplify[{2 x, 2 y, -1 + a, 1 + a}/(1 + a)]]]; 


isotrop[z_] := Module[{x, y, a},
    If[z == Null, Print["Argument of isotrop is Null!"]; Return[]];
    If[z == ComplexInfinity, Return[{0, 0, 1, 1}]];
    x = Re[z];
     y = Im[z];
    a = Abs[z]^2;
    Return[FullSimplify[{2 x, 2 y, -1 + a, 1 + a}]]];
    

invisotrop[a_List] := Module[{b},If[Length[a]\[NotEqual]4,Print["Argument of 
    invisotrop has incorrect dimension!"];Return[]];
    If[a[[4]]\[Equal]0 || Simplify[Chop[pssp[a,a]]]!=0,
      Print["Incorrect argument of invisotrop!"];Return[]];
    b=-a[[3]]+a[[4]];
    If[b\[Equal]0,Return[ComplexInfinity]];
    Return[a[[1]]/b+I*a[[2]]/b]]

(*The Riemannian sphere visualized as the unit sphere in the euclidean 3-space *)    

rs2[z_]:= Take[isotrop1[z], 3]
rs2point[z_] := Point[rs2[z]]
SetAttributes[rs2point, Listable]    

rs2pointplot[z_, d_:0.01] := Show[Graphics3D[{PointSize[d],rs2point[z]}]]
SetAttributes[rs2pointplot,Listable]

(* The elements of the Moebius group *)

g0[a_,b_,c_,d_][z_] :=Module[{},
    If[z\[Equal]infy,Return[a/c]];
    (a z+b)/(c z+d)]; 
    
g1[a_,b_,c_,d_][z_] :=Module[{},
    If[z\[Equal]infy,Return[a/c]];
    (a Conjugate[z]+b)/(c Conjugate[z]+d)];
    
 (* The cross ratio of a throw in the Riemannian sphere *)   
 
 SetAttributes[crossratio, Orderless];
    
    crossratio[p1_List,p2_List]:= Module[{p=Join[p1,p2]},
    If[Length[p] \[NotEqual] 4 , Print["Incorrect Argument!"];Return[]];
    If[Length[Union[p]]<3, Print["Less than 3 points!"];Return[]];
    If[p[[1]] \[Equal]p[[4]] || p[[2]] \[Equal]p[[3]] , 
      Return[ ComplexInfinity]];
    If[p[[1]] \[Equal]p[[3]] || p[[2]] \[Equal]p[[4]], Return[0]];
    If[p[[1]] \[Equal]p[[2]] || p[[4]] \[Equal]p[[3]], Return[1]];
    If[p[[1]] \[Equal] ComplexInfinity, 
      Return[(p[[2]]-p[[4]])*(p[[2]]-p[[3]])^-1]];
    If[p[[2]] \[Equal] ComplexInfinity, 
      Return[(p[[1]]-p[[3]])*(p[[1]]-p[[4]])^-1]];
    If[p[[3]] \[Equal] ComplexInfinity, 
      Return[(p[[2]]-p[[4]])*(p[[1]]-p[[4]])^-1]];
    If[p[[4]] \[Equal] ComplexInfinity, 
      Return[(p[[1]]-p[[3]])*(p[[2]]-p[[3]])^-1]];
    (p[[1]]-
          p[[3]])*(p[[1]]-p[[4]])^-1*((p[[2]]-p[[3]])*(p[[2]]-p[[4]])^-1)^-1];
          
   crvalues[z_]:=Union[FullSimplify[{z, 1-z, 1/z, 1 - 1/z, 1/(1-z), z/(z-1)}]];

(* Projective scales *)

scale[b_,c_,d_][z_] := crossratio[{z,b},{c,d}]

(* Pseudo-orthogonal frame adapted to a 0-sphere *)

rspsframepp[z1_,z2_] := Module[{
       m= Table[null[],{4}],mv, mw},
    		If[z1\[Equal]z2, Print["Incorrect arguments!"];Return[]];
    m[[3]]=isotrop[z1]-isotrop[z2];
    m[[4]]=isotrop[z1]+isotrop[z2];
    mv = psCross[stb[1],m[[3]],m[[4]]];
    mw = psCross[stb[2],m[[3]],m[[4]]];
    If[mv\[Equal] null[],m[[1]]=mw, m[[1]]=mv,m[[1]]=mv];
    m[[2]] = -psCross[m[[1]],m[[3]],m[[4]]];
    Return[Table[normalize[m[[i]]], {i,4}]]]	

(* The orthogonal subspace *)    
rsvspacepp[z1_,z2_] := Module[{psf},
    psf = rspsframepp[z1,z2];
    If[psf \[Equal]Null, Return[Null]];
    {psf[[1]], psf[[2]]}]  
    
(* The equivariant operator *)      
    
    rsoperatorpp[a_List, b_List] := Module[{av, bv, i, j,k},
		av = rsvspacepp[a[[1]], a[[2]]];
	bv = rsvspacepp[b[[1]], b[[2]]];
    If[av \[Equal] Null||bv\[Equal]Null, Return[]];
    If[a\[Equal]b, Return[IdentityMatrix[2]]];
    		Table[Sum[pssp[av[[j]],bv[[k]]]*pssp[av[[i]],bv[[k]]],{k,2}],
      			{i,2},{j,2}]]
 
 (* Circle through 3 points *)     			
      			
 rscenter3pts[a1_, a2_, a3_] := 
  ((-a2 + a3)*Abs[a1]^2 + (a1 - a3)*Abs[a2]^2 + 
    (-a1 + a2)*Abs[a3]^2)/((-a2 + a3)*Conjugate[a1] + 
    (a1 - a3)*Conjugate[a2] + (-a1 + a2)*Conjugate[a3]);

rsradius3pts[a1_, a2_, a3_] := 
  (Abs[a1 - a2]*Abs[a1 - a3]*Abs[a2 - a3])/
   Abs[(a2 - a3)*Conjugate[a1] + (-a1 + a3)*Conjugate[a2] + 
     (a1 - a2)*Conjugate[a3]]    
     
rscircle3pts[a1_,a2_,a3_][t_]:= Module[{m,r,cli},
    If[Chop[a1-a2]\[Equal]0|| Chop[a3-a2]\[Equal]0|| Chop[a1-a3]\[Equal]0, 
      Print["Less than 3 points!"];Goto[cli]];
    m=rscenter3pts[a1,a2,a3];r=rsradius3pts[a1,a2,a3];
    If[Chop[1/r]\[Equal]0,Goto[cli]];
    Return[{Re[m],Im[m]}+r*{Cos[t],Sin[t]}];
    Label[cli];
    {Re[a1 +t*(a2-a1)],Im[a1 +t*(a2-a1)]}]       			

rsplotcircle3pts[a1_,a2_,a3_, x_:0, y_:2*Pi,opt___]:= Module[{m,r,cli},
    If[Chop[a1-a2]\[Equal]0|| Chop[a3-a2]\[Equal]0|| Chop[a1-a3]\[Equal]0, 
      Print["Less than 3 points!"];Goto[cli]];
    m=rscenter3pts[a1,a2,a3];r=rsradius3pts[a1,a2,a3];
    If[Chop[1/r]\[Equal]0,Goto[cli]];
    Return[
      ParametricPlot[{Re[m],Im[m]}+r*{Cos[t],Sin[t]},{t,x,y},
        AspectRatio->Automatic,opt]];
    Label[cli];
    ParametricPlot[{Re[a1 +t*(a2-a1)],Im[a1 +t*(a2-a1)]}, {t,x,y},opt]]
    
rsvector3pts[z1_, z2_, z3_] := psCross[isotrop[z1], isotrop[z2], isotrop[z3]]

(* Circle as Function of center and radius *)

rscirclevector[m_, r_] := {2*Re[m],2*Im[m],-1-r^2+Abs[m]^2, 1-r^2+Abs[m]^2}/(2*r)     
    
rsvcenter[a_List]:= Module[{vm},
    If[Length[a] \[NotEqual]  4, Print[Length[a], " incorrect dimension!"]; 
      Return[]];
    vm =vcenter[a];
    If[vm == Null, Return[]];
    If[vm[[1]] \[Equal]Infinity, Return[ComplexInfinity]];
    vm[[1]] + I*vm[[2]]]
    
(* The Coxeter invariant *)

rsCoxeter[m_,m1_,r_,r1_] := (r^2 +r1^2 -Abs[m-m1]^2)/(2*r*r1)    

(* The homomorphism sl2Ctoso31 *)

sl2Ctoso31[a_, b_, c_, d_] := Transpose[{{-1 + 2*Im[b]*Im[c] + 2*Re[a]*Re[d], 
    -2*Im[c]*Re[b] + 2*Im[a]*Re[d], Im[a]*Im[b] - Im[c]*Im[d] + 
     Re[a]*Re[b] - Re[c]*Re[d], Im[a]*Im[b] + Im[c]*Im[d] + 
     Re[a]*Re[b] + Re[c]*Re[d]}, {2*Im[d]*Re[a] - 2*Im[c]*Re[b], 
    1 - 2*Im[b]*Im[c] + 2*Im[a]*Im[d], Im[b]*Re[a] - Im[a]*Re[b] - 
     Im[d]*Re[c] + Im[c]*Re[d], Im[b]*Re[a] - Im[a]*Re[b] + 
     Im[d]*Re[c] - Im[c]*Re[d]}, {Im[a]*Im[c] - Im[b]*Im[d] + 
     Re[a]*Re[c] - Re[b]*Re[d], (-Im[c])*Re[a] + Im[d]*Re[b] + 
     Im[a]*Re[c] - Im[b]*Re[d], (1/2)*(Abs[a]^2 + Abs[d]^2 - Im[b]^2 - 
      Im[c]^2 - Re[b]^2 - Re[c]^2), 
    (1/2)*(Abs[a]^2 + Abs[c]^2 - Im[b]^2 - Im[d]^2 - Re[b]^2 - 
      Re[d]^2)}, {Im[a]*Im[c] + Im[b]*Im[d] + Re[a]*Re[c] + 
     Re[b]*Re[d], (-Im[c])*Re[a] - Im[d]*Re[b] + Im[a]*Re[c] + 
     Im[b]*Re[d], (1/2)*(Abs[a]^2 + Abs[b]^2 - Im[c]^2 - Im[d]^2 - 
      Re[c]^2 - Re[d]^2), (1/2)*(Abs[a]^2 + Abs[b]^2 + Abs[c]^2 + 
      Abs[d]^2)}}]        
      			

 End[]
  

Protect[infy, isotrop1,isotrop, invisotrop, rs2, rs2point, rs2pointplot, g0, g1, crossratio,
 crvalues, scale, rsoperatorpp, rspsframepp, rsvspacepp, rsradius3pts, rscenter3pts, 
 rscircle3pts, rsplotcircle3pts,
 rsvector3pts, rscirclevector, rsvcenter, rsCoxeter, sl2Ctoso31]

EndPackage[]